@@ -3,7 +3,7 @@ import sys
 import time
 import json
 from typing import Any, Callable, Dict, List, Optional, Union, Tuple
-
+from threading import Lock
 import grpc
 from formant.protos.agent.v1 import agent_pb2, agent_pb2_grpc
 from formant.protos.model.v1 import (
@@ -33,6 +33,7 @@ DEFAULT_SEVERITY_TYPE = "info"  # type: Literal["info"]
 ALLOWED_IMAGE_CONTENT_TYPES = ["image/jpg", "image/jpeg", "image/png"]
 ALLOWED_SEVERITY_TYPES = ["info", "warning", "critical", "error"]
 DEFUALT_LOCALIZATION_STEAM_NAME = "localization"
+DEFAULT_THROTTLE_HZ = 5
 
 
 def current_timestamp():
@@ -138,7 +139,7 @@ class Client:
 
         self.ignore_throttled = ignore_throttled
         self.ignore_unavailable = ignore_unavailable
-
+        self._lock = Lock()
         self._agent_url = agent_url  # type: str
         if local_dev == True:
             self._agent_url = os.getenv(
@@ -165,16 +166,28 @@ class Client:
         self._setup_agent_communication()
 
     def get_localization_manager(
-        self, stream_name=DEFUALT_LOCALIZATION_STEAM_NAME,  # type: str
+        self,
+        stream_name=DEFUALT_LOCALIZATION_STEAM_NAME,  # type: str
+        throttle_hz=DEFAULT_THROTTLE_HZ,  # type:float
     ):
         # type: (...) -> LocalizationManager
         validate_type(stream_name, [str])
         if self._localization_manager_cache.get(stream_name, None) is None:
             self._localization_manager_cache[stream_name] = LocalizationManager(
-                stream_name, self
+                stream_name, self, throttle_hz
             )
         return self._localization_manager_cache[stream_name]
 
+    @handle_agent_exceptions
+    def get_agent_id(self):
+        return self.get_agent_configuration().id
+
+    @handle_agent_exceptions
+    def get_agent_configuration(self):
+        return self.agent_stub.GetAgentConfiguration(
+            agent_pb2.GetAgentConfigurationRequest()
+        ).configuration
+
     # Ingesting telemetry datapoints
 
     @handle_agent_exceptions
@@ -330,7 +343,9 @@ class Client:
             numeric_set.numerics.extend(
                 [
                     math_pb2.NumericSetEntry(
-                        value=v[0], label=k, unit=v[1] if v[1] else None,
+                        value=v[0],
+                        label=k,
+                        unit=v[1] if v[1] else None,
                     )
                 ]
             )
@@ -585,13 +600,17 @@ class Client:
         :param stream_filter: A list of stream names. The provided callback
             is only called for datapoints whose stream name is in this list
         """
+        with self._lock:
 
-        def create_stream():
-            return self._get_telemetry_listener_stream(stream_filter=stream_filter)
+            def create_stream():
+                return self._get_telemetry_listener_stream(stream_filter=stream_filter)
 
-        self._telemetry_listener_callback_streams[f] = CancellableStreamThread(
-            f, create_stream, self.logger, ignore_unavailable=self.ignore_unavailable
-        )
+            self._telemetry_listener_callback_streams[f] = CancellableStreamThread(
+                f,
+                create_stream,
+                self.logger,
+                ignore_unavailable=self.ignore_unavailable,
+            )
 
     def unregister_telemetry_listener_callback(
         self, f  # type: Callable[[], None]
@@ -602,9 +621,10 @@ class Client:
 
         :param f: The telemetry loopback callback to be unregistered
         """
-        stream = self._telemetry_listener_callback_streams.pop(f, None)
-        if stream is not None:
-            stream.cancel()
+        with self._lock:
+            stream = self._telemetry_listener_callback_streams.pop(f, None)
+            if stream is not None:
+                stream.cancel()
 
     # Ingesting transform frames
 
@@ -756,7 +776,9 @@ class Client:
         :param datapoint: A datapoint related to the command
         """
         response = commands_pb2.CommandResponse(
-            request_id=request_id, success=success, datapoint=datapoint,
+            request_id=request_id,
+            success=success,
+            datapoint=datapoint,
         )
         request = agent_pb2.SendCommandResponseRequest(response=response)
         self.agent_stub.SendCommandResponse(request)
@@ -777,20 +799,22 @@ class Client:
         :param command_filter: A list of command names. The provided callback
             is only executed on commands whose names are in this list
         """
+        with self._lock:
 
-        def create_stream():
-            return self._get_command_request_stream(command_filter)
+            def create_stream():
+                return self._get_command_request_stream(command_filter)
 
-        self._command_request_callback_streams[f] = CancellableStreamThread(
-            f,
-            create_stream,
-            self.logger,
-            attribute="request",
-            ignore_unavailable=self.ignore_unavailable,
-        )
+            self._command_request_callback_streams[f] = CancellableStreamThread(
+                f,
+                create_stream,
+                self.logger,
+                attribute="request",
+                ignore_unavailable=self.ignore_unavailable,
+            )
 
     def unregister_command_request_callback(
-        self, f,  # type: Callable[[commands_pb2.CommandRequest], None]
+        self,
+        f,  # type: Callable[[commands_pb2.CommandRequest], None]
     ):
         # type: (...) -> None
         """
@@ -798,9 +822,10 @@ class Client:
 
         :param f: The command request callback to be unregistered
         """
-        stream = self._command_request_callback_streams.pop(f, None)
-        if stream is not None:
-            stream.cancel()
+        with self._lock:
+            stream = self._command_request_callback_streams.pop(f, None)
+            if stream is not None:
+                stream.cancel()
 
     # Teleop
 
@@ -822,16 +847,18 @@ class Client:
             is only exectued on control datapoints whose names are in this list
         """
 
-        def create_stream():
-            return self._get_teleop_stream(stream_filter)
+        with self._lock:
 
-        self._teleop_callback_streams[f] = CancellableStreamThread(
-            f,
-            create_stream,
-            self.logger,
-            attribute="control_datapoint",
-            ignore_unavailable=self.ignore_unavailable,
-        )
+            def create_stream():
+                return self._get_teleop_stream(stream_filter)
+
+            self._teleop_callback_streams[f] = CancellableStreamThread(
+                f,
+                create_stream,
+                self.logger,
+                attribute="control_datapoint",
+                ignore_unavailable=self.ignore_unavailable,
+            )
 
     def unregister_teleop_callback(
         self, f  # type: Callable[[datapoint_pb2.ControlDatapoint], None]
@@ -842,9 +869,10 @@ class Client:
 
         :param f: The teleop callback to be unregistered
         """
-        stream = self._teleop_callback_streams.pop(f, None)
-        if stream is not None:
-            stream.cancel()
+        with self._lock:
+            stream = self._teleop_callback_streams.pop(f, None)
+            if stream is not None:
+                stream.cancel()
 
     @handle_agent_exceptions
     def get_teleop_info(self):
@@ -856,7 +884,8 @@ class Client:
         return self.agent_stub.GetTeleopInfo(request)
 
     def register_teleop_heartbeat_callback(
-        self, f,  # type: Callable[[agent_pb2.GetTeleopHeartbeatStreamResponse], None]
+        self,
+        f,  # type: Callable[[agent_pb2.GetTeleopHeartbeatStreamResponse], None]
     ):
         # type : (...) -> None
         """
@@ -867,13 +896,17 @@ class Client:
 
         :param f: A callback that will be called when a heartbeat is received
         """
+        with self._lock:
 
-        def create_stream():
-            return self._get_teleop_heartbeat_stream()
+            def create_stream():
+                return self._get_teleop_heartbeat_stream()
 
-        self._teleop_heartbeat_callback_streams[f] = CancellableStreamThread(
-            f, create_stream, self.logger, ignore_unavailable=self.ignore_unavailable
-        )
+            self._teleop_heartbeat_callback_streams[f] = CancellableStreamThread(
+                f,
+                create_stream,
+                self.logger,
+                ignore_unavailable=self.ignore_unavailable,
+            )
 
     def unregister_teleop_heartbeat_callback(
         self, f  # type: Callable[[], None]
@@ -884,9 +917,10 @@ class Client:
 
         :param f: The teleop heartbeat callback to be unregistered
         """
-        stream = self._teleop_heartbeat_callback_streams.pop(f, None)
-        if stream is not None:
-            stream.cancel()
+        with self._lock:
+            stream = self._teleop_heartbeat_callback_streams.pop(f, None)
+            if stream is not None:
+                stream.cancel()
 
     # Custom data channel methods
 
@@ -914,26 +948,34 @@ class Client:
         :param channel_name_filter: An optional allow list of custom channel names
             for this callback
         """
+        with self._lock:
 
-        def create_stream():
-            return self._get_custom_data_channel_message_stream(
-                channel_name_filter=channel_name_filter
+            def create_stream():
+                return self._get_custom_data_channel_message_stream(
+                    channel_name_filter=channel_name_filter
+                )
+
+            self._custom_data_channel_message_callback_streams[
+                f
+            ] = CancellableStreamThread(
+                f,
+                create_stream,
+                self.logger,
+                ignore_unavailable=self.ignore_unavailable,
             )
 
-        self._custom_data_channel_message_callback_streams[f] = CancellableStreamThread(
-            f, create_stream, self.logger, ignore_unavailable=self.ignore_unavailable
-        )
-
     def unregister_custom_data_channel_message_callback(
-        self, f,  # type: Callable[[], None]
+        self,
+        f,  # type: Callable[[], None]
     ):
         # type : (...) -> None
         """
         :param f: The custom data channel message callback to be unregistered
         """
-        stream = self._custom_data_channel_message_callback_streams.pop(f, None)
-        if stream is not None:
-            stream.cancel()
+        with self._lock:
+            stream = self._custom_data_channel_message_callback_streams.pop(f, None)
+            if stream is not None:
+                stream.cancel()
 
     def custom_data_channel_request_handler(
         self, channel_name  # type: str
@@ -1113,7 +1155,8 @@ class Client:
 
         :param f: The configuration update callback to be registered
         """
-        self._config_update_callbacks.append(f)
+        with self._lock:
+            self._config_update_callbacks.append(f)
 
     def unregister_config_update_callback(self, f):
         # type: (Callable) -> None
@@ -1123,7 +1166,8 @@ class Client:
 
         :param f: The configuration update callback to be unregistered
         """
-        self._config_update_callbacks.remove(f)
+        with self._lock:
+            self._config_update_callbacks.remove(f)
 
     # Internal methods
 
@@ -1379,7 +1423,8 @@ class Client:
 
     @handle_agent_exceptions
     def get_intervention_response(
-        self, request_id,  # type: str
+        self,
+        request_id,  # type: str
     ):
         # type : (...) -> agent_pb2.GetInterventionResponse
 
