@@ -15,14 +15,19 @@ class Odometry:
         )
 
     @classmethod
-    def from_xy_direction(cls, x, y, direction):
+    def from_xy_direction(
+        cls,
+        x,  # type: float
+        y,  # type: float
+        direction,  # type:float
+    ):
         return cls(pose=Transform.from_xy_direction(x, y, direction))
 
     def __init__(
         self,
-        pose: Transform = Transform(),
-        twist: Twist = Twist(),
-        transform_to_world: Transform = Transform(),
+        pose=Transform(),  # type: Transform
+        twist=Twist(),  # type: Twist
+        transform_to_world=Transform(),  # type: Transform
     ):
         self.pose = pose
         self.twist = twist
