@@ -16,14 +16,23 @@ class Vector3:
         return cls(point.x, point.y, point.z)
 
     @classmethod
-    def from_xy(cls, x: float, y: float):
+    def from_xy(
+        cls,
+        x,  # type: float
+        y,  # type: float
+    ):
         return cls(x, y, 0)
 
     @classmethod
     def identity(cls):
         return cls(0, 0, 0)
 
-    def __init__(self, x: float, y: float, z: float):
+    def __init__(
+        self,
+        x,  # type: float
+        y,  # type: float
+        z,  # type: float
+    ):
         self.x = x
         self.y = y
         self.z = z
