@@ -1,19 +1,23 @@
+from typing import Optional, List, Dict
 import ntpath
 import os
 import requests
 import sys
 import time
 import datetime
+from dateutil.tz import tzutc
 import copy
 
 FORMANT_REQUEST_ON_DEMAND_DATA_COMMAND_NAME = "formant.demand_data"
 
 
 def get_current_isodate():
-    return datetime.datetime.now(tz=datetime.timezone.utc).isoformat()
+    return datetime.datetime.now(tz=tzutc()).isoformat()
 
 
-def get_timestamp_str(dt: datetime.datetime):
+def get_timestamp_str(
+    dt,  # type: datetime.datetime
+):
     return str(int(dt.timestamp()))
 
 
@@ -51,8 +55,21 @@ class Client:
             "Content-Type": "application/json",
             "App-ID": "formant/python-cloud-sdk",
         }
+
         self._token = None
         self._token_expiry = 0
+        self._organization_id = None
+        self._user_id = None
+
+    def get_user_id(self):
+        if self._user_id is None:
+            self._authenticate()
+        return self._user_id
+
+    def get_organization_id(self):
+        if self._organization_id is None:
+            self._authenticate()
+        return self._organization_id
 
     def ingest(self, params):
         """Administrator credentials required.
@@ -273,9 +290,9 @@ class Client:
 
     def demand_device_data(
         self,
-        device_id: str,
-        start: datetime.datetime,
-        end: datetime.datetime,
+        device_id,  # type: str
+        start,  # type: datetime.datetime
+        end,  # type: datetime.datetime
     ):
         params = {
             "deviceId": device_id,
@@ -486,6 +503,76 @@ class Client:
 
         return self._authenticate_request(call)
 
+    def get_annotation_templates(self):
+        def call(token):
+            headers = self._headers.copy()
+            headers["Authorization"] = "Bearer %s" % token
+            response = requests.get(
+                "%s/annotation-templates/" % self._admin_api,
+                headers=headers,
+            )
+            return response.json()
+
+        return self._authenticate_request(call)
+
+    def post_annotation(self, params):
+        def call(token):
+            self._add_user_id_to_params(params)
+            self._add_organization_id_to_params(params)
+            params_stripped = _strip_none_values(params)
+            headers = self._headers.copy()
+            headers["Authorization"] = "Bearer %s" % token
+            response = requests.post(
+                "%s/annotations/" % (self._admin_api),
+                headers=headers,
+                json=params_stripped,
+            )
+            return response.json()
+
+        return self._authenticate_request(call)
+
+    def query_events(self, params):
+        def call(token):
+            headers = self._headers.copy()
+            headers["Authorization"] = "Bearer %s" % token
+            response = requests.post(
+                "%s/events/query" % (self._admin_api),
+                headers=headers,
+                json=params,
+            )
+            return response.json()
+
+        return self._authenticate_request(call)
+
+    def query_annotations(
+        self,
+        start_time,  # type: datetime.datetime
+        device_ids=None,  # type: Optional[List[str]]
+        tags=None,  # type: Optional[Dict[str,List[str]]]
+        annotation_template_ids=None,  # type: Optional[List[str]]
+        message=None,  # type:str
+        keyword=None,  # type: str
+        end_time=datetime.datetime.now(tz=tzutc()),  # type: datetime.datetime
+        params={},  # type: Dict
+    ):
+        params["tags"] = tags
+        params["start"] = start_time.isoformat()
+        params["end"] = end_time.isoformat()
+        params["deviceIds"] = device_ids
+        params["annotationTemplateIds"] = annotation_template_ids
+        params["message"] = message
+        params["keyword"] = keyword
+        params["eventTypes"] = ["annotation"]
+
+        params_stripped = _strip_none_values(params)
+        return self.query_events(params_stripped)
+
+    def _add_user_id_to_params(self, params):
+        params["userId"] = self.get_user_id()
+
+    def _add_organization_id_to_params(self, params):
+        params["organizationId"] = self.get_organization_id()
+
     def _authenticate(self):
         payload = {
             "email": self._email,
@@ -504,6 +591,7 @@ class Client:
         self._token_expiry = int(time.time()) + 3530
         self._token = result["authentication"]["accessToken"]
         self._organization_id = result["authentication"]["organizationId"]
+        self._user_id = result["authentication"]["userId"]
 
     def _authenticate_request(self, call):
         if self._token is None or self._token_expiry < int(time.time()):
@@ -516,8 +604,12 @@ class Client:
                 try:
                     return call(self._token)
                 except requests.exceptions.HTTPError as error:
-                    print(error.response.text, file=sys.stderr)
+                    sys.stderr.write("%s\n" % error.response.text)
                     raise error
             else:
-                print(error.response.text, file=sys.stderr)
+                sys.stderr.write("%s\n" % error.response.text)
                 raise error
+
+
+def _strip_none_values(dict):
+    return {k: v for k, v in dict.items() if v is not None}
